@@ -1,3 +1,12 @@
+#     (\
+#     (  \  /(o)\     caw!
+#     (   \/  ()/ /)
+#      (   `;.))'".) 
+#       `(/////.-'
+#    =====))=))===() 
+#      ///'       
+#     //
+#    '            
 from datetime import datetime
 import logging
 import os
@@ -20,9 +29,9 @@ class Database(object):
     def close(self):
         self.conn.close()
     
-    def execute(self, sql, commit=False):
+    def execute(self, sql, params=None, commit=False):
         cursor = self.conn.cursor()
-        res = cursor.execute(sql)
+        res = cursor.execute(sql, params or ())
         if commit:
             self.conn.commit()
         logger.debug(sql)
@@ -38,13 +47,13 @@ class Database(object):
 
         for field in model_class._meta.fields.values():
             columns.append(field.to_sql())
+
+        query = framing % (model_class._meta.db_table, ', '.join(columns))
         
-        self.execute(
-            framing % (model_class._meta.db_table, ', '.join(columns)), True
-        )
+        self.execute(query, commit=True)
     
     def drop_table(self, model_class):
-        self.execute('DROP TABLE %s;' % model_class._meta.db_table, True)
+        self.execute('DROP TABLE %s;' % model_class._meta.db_table, commit=True)
 
 
 database = Database(DATABASE_NAME)
@@ -114,21 +123,20 @@ def mark_query_dirty(func):
 
 class BaseQuery(object):
     operations = {
-        'lt': '< %s',
-        'lte': '<= %s',
-        'gt': '> %s',
-        'gte': '>= %s',
-        'eq': '= %s',
-        'in': 'IN (%s)',
-        'is': 'IS %s',
-        'icontains': "LIKE '%%%s%%' ESCAPE '\\'",
-        'contains': "GLOB '*%s*'",
+        'lt': '< ?',
+        'lte': '<= ?',
+        'gt': '> ?',
+        'gte': '>= ?',
+        'eq': '= ?',
+        'in': 'IN (%s)', # special-case to list q-marks
+        'is': 'IS ?',
+        'icontains': "LIKE ? ESCAPE '\\'", # surround param with %'s
+        'contains': "GLOB ?", # surround param with *'s
     }
     query_separator = '__'
     requires_commit = True
     
-    def __init__(self, database, model):
-        self.database = database
+    def __init__(self, model):
         self.model = model
         self.query_context = model
         self._where = {}
@@ -145,10 +153,13 @@ class BaseQuery(object):
 
             field = self.query_context._meta.get_field_by_name(lhs)
             if op == 'in':
-                lookup_value = ','.join([field.lookup_value(op, o) for o in rhs])
+                lookup_value = [field.lookup_value(op, o) for o in rhs]
+                operation = self.operations[op] % (','.join(['?' for v in lookup_value]))
             else:
                 lookup_value = field.lookup_value(op, rhs)
-            parsed[field.name] = self.operations[op] % lookup_value
+                operation = self.operations[op]
+            
+            parsed[field.name] = (operation, lookup_value)
         
         return parsed
     
@@ -166,9 +177,9 @@ class BaseQuery(object):
         return self
     
     @mark_query_dirty
-    def join(self, model):
+    def join(self, model, join_type=None):
         if self.query_context._meta.rel_exists(model):
-            self._joins.append(model)
+            self._joins.append((model, join_type))
             self.query_context = model
         else:
             raise AttributeError('No foreign key found between %s and %s' % \
@@ -191,12 +202,13 @@ class BaseQuery(object):
 
         joins = list(self._joins)
         if self._where or len(joins):
-            joins.insert(0, self.model)
+            joins.insert(0, (self.model, None))
         
         where_with_alias = []
+        where_data = []
         computed_joins = []
 
-        for i, model in enumerate(joins):
+        for i, (model, join_type) in enumerate(joins):
             if alias_required:
                 alias_count += 1
                 alias_map[model] = 't%d' % alias_count
@@ -208,11 +220,13 @@ class BaseQuery(object):
                     if name == '__raw__':
                         where_with_alias.append(lookup)
                     else:
-                        where_with_alias.append('%s %s' % (
-                            self.combine_field(alias_map[model], name), lookup))
+                        operation, value = lookup
+                        where_data.append(value)
+                        combined = self.combine_field(alias_map[model], name)
+                        where_with_alias.append('%s %s' % (combined, operation))
             
             if i > 0:
-                from_model = joins[i-1]
+                from_model = joins[i-1][0]
                 field = from_model._meta.get_related_field_for_model(model)
                 if field:
                     left_field = field.name
@@ -222,10 +236,11 @@ class BaseQuery(object):
                     left_field = 'id'
                     right_field = field.name
                 
-                if field.null and model not in self._where:
-                    join_type = 'LEFT OUTER'
-                else:
-                    join_type = 'INNER'
+                if join_type is None:
+                    if field.null and model not in self._where:
+                        join_type = 'LEFT OUTER'
+                    else:
+                        join_type = 'INNER'
                 
                 computed_joins.append(
                     '%s JOIN %s AS %s ON %s = %s' % (
@@ -237,11 +252,12 @@ class BaseQuery(object):
                     )
                 )
         
-        return computed_joins, where_with_alias, alias_map
+        return computed_joins, where_with_alias, where_data, alias_map
     
     def raw_execute(self):
-        query = self.sql()
-        result = self.database.execute(query, self.requires_commit)
+        query, params = self.sql()
+        db = self.model._meta.get_database()
+        result = db.execute(query, params, self.requires_commit)
         return result
 
 
@@ -251,7 +267,7 @@ class SelectQuery(BaseQuery):
     """
     requires_commit = False
     
-    def __init__(self, database, model, query=None):
+    def __init__(self, model, query=None):
         """
         Allow a string or a dictionary keyed by model->fields
         
@@ -265,7 +281,7 @@ class SelectQuery(BaseQuery):
         self._pagination = None # return all by default
         self._distinct = False
         self._qr = None
-        super(SelectQuery, self).__init__(database, model)
+        super(SelectQuery, self).__init__(model)
     
     @mark_query_dirty
     def paginate(self, page_num, paginate_by=20):
@@ -283,7 +299,8 @@ class SelectQuery(BaseQuery):
         else:
             self.query = 'COUNT(id)'
         
-        res = self.database.execute(self.sql())
+        db = self.model._meta.get_database()
+        res = db.execute(*self.sql())
         
         self.query = tmp_query
         self._pagination = tmp_pagination
@@ -340,10 +357,11 @@ class SelectQuery(BaseQuery):
             raise TypeError('Unknown type encountered parsing select query')
     
     def sql(self):
-        joins, where, alias_map = self.compile_where()
+        joins, where, where_data, alias_map = self.compile_where()
         
         table = self.model._meta.db_table
 
+        params = []
         group_by = []
         
         if self.use_aliases():
@@ -380,6 +398,11 @@ class SelectQuery(BaseQuery):
             pieces.append(joins)
         if where:
             pieces.append('WHERE %s' % where)
+            for clause in where_data:
+                if isinstance(clause, (tuple, list)):
+                    params.extend(clause)
+                else:
+                    params.append(clause)
         if group_by:
             pieces.append('GROUP BY %s' % group_by)
         if having:
@@ -392,7 +415,7 @@ class SelectQuery(BaseQuery):
                 page -= 1
             pieces.append('LIMIT %d OFFSET %d' % (paginate_by, page * paginate_by))
         
-        return ' '.join(pieces)
+        return ' '.join(pieces), params
     
     def execute(self):
         if self._dirty:
@@ -411,31 +434,40 @@ class UpdateQuery(BaseQuery):
     """
     Model.update(field=val, field2=val2).where(some_field=some_val)
     """
-    def __init__(self, database, model, **kwargs):
+    def __init__(self, model, **kwargs):
         self.update_query = kwargs
-        super(UpdateQuery, self).__init__(database, model)
+        super(UpdateQuery, self).__init__(model)
     
     def parse_update(self):
-        sets = []
+        sets = {}
         for k, v in self.update_query.iteritems():
             field = self.model._meta.get_field_by_name(k)
-            sets.append('%s=%s' % (k, field.lookup_value(None, v)))
+            sets[field.name] = field.lookup_value(None, v)
         
-        return ', '.join(sets)
+        return sets
     
     def sql(self):
-        joins, where, alias_map = self.compile_where()
+        joins, where, where_data, alias_map = self.compile_where()
         set_statement = self.parse_update()
+
+        params = []
+        update_params = []
+
+        for k, v in set_statement.iteritems():
+            params.append(v)
+            update_params.append('%s=?' % k)
         
-        update = 'UPDATE %s SET %s' % (self.model._meta.db_table, set_statement)
+        update = 'UPDATE %s SET %s' % (
+            self.model._meta.db_table, ', '.join(update_params))
         where = ' AND '.join(where)
         
         pieces = [update]
         
         if where:
             pieces.append('WHERE %s' % where)
+            params.extend(where_data)
         
-        return ' '.join(pieces)
+        return ' '.join(pieces), params
     
     def join(self, *args, **kwargs):
         raise AttributeError('Update queries do not support JOINs in sqlite')
@@ -450,7 +482,9 @@ class DeleteQuery(BaseQuery):
     Model.delete().where(some_field=some_val)
     """
     def sql(self):
-        joins, where, alias_map = self.compile_where()
+        joins, where, where_data, alias_map = self.compile_where()
+
+        params = []
         
         delete = 'DELETE FROM %s' % (self.model._meta.db_table)
         where = ' AND '.join(where)
@@ -459,8 +493,9 @@ class DeleteQuery(BaseQuery):
         
         if where:
             pieces.append('WHERE %s' % where)
+            params.extend(where_data)
         
-        return ' '.join(pieces)
+        return ' '.join(pieces), params
     
     def join(self, *args, **kwargs):
         raise AttributeError('Update queries do not support JOINs in sqlite')
@@ -474,9 +509,9 @@ class InsertQuery(BaseQuery):
     """
     Model.insert(field=val, field2=val2)
     """
-    def __init__(self, database, model, **kwargs):
+    def __init__(self, model, **kwargs):
         self.insert_query = kwargs
-        super(InsertQuery, self).__init__(database, model)
+        super(InsertQuery, self).__init__(model)
     
     def parse_insert(self):
         cols = []
@@ -484,7 +519,7 @@ class InsertQuery(BaseQuery):
         for k, v in self.insert_query.iteritems():
             field = self.model._meta.get_field_by_name(k)
             cols.append(k)
-            vals.append(str(field.lookup_value(None, v)))
+            vals.append(field.lookup_value(None, v))
         
         return cols, vals
     
@@ -492,9 +527,9 @@ class InsertQuery(BaseQuery):
         cols, vals = self.parse_insert()
         
         insert = 'INSERT INTO %s (%s) VALUES (%s)' % (
-            self.model._meta.db_table, ','.join(cols), ','.join(vals))
+            self.model._meta.db_table, ','.join(cols), ','.join('?' for v in vals))
         
-        return insert
+        return insert, vals
     
     def where(self, *args, **kwargs):
         raise AttributeError('Insert queries do not support WHERE clauses')
@@ -532,7 +567,7 @@ class Field(object):
         return '"%s" %s' % (self.name, rendered)
     
     def db_value(self, value):
-        return value or "NULL"
+        return value
     
     def python_value(self, value):
         return value
@@ -553,10 +588,12 @@ class CharField(Field):
         return value[:self.attributes['max_length']]
     
     def lookup_value(self, lookup_type, value):
-        if lookup_type in ('contains', 'icontains'):
-            return self.db_value(value)
+        if lookup_type == 'contains':
+            return '*%s*' % self.db_value(value)
+        elif lookup_type == 'icontains':
+            return '%%%s%%' % self.db_value(value)
         else:
-            return '"%s"' % self.db_value(value)
+            return self.db_value(value)
     
 
 class TextField(Field):
@@ -566,10 +603,12 @@ class TextField(Field):
         return value or ''
     
     def lookup_value(self, lookup_type, value):
-        if lookup_type in ('contains', 'icontains'):
-            return self.db_value(value)
+        if lookup_type == 'contains':
+            return '*%s*' % self.db_value(value)
+        elif lookup_type == 'icontains':
+            return '%%%s%%' % self.db_value(value)
         else:
-            return '"%s"' % self.db_value(value)
+            return self.db_value(value)
 
 
 class DateTimeField(Field):
@@ -581,9 +620,7 @@ class DateTimeField(Field):
             return datetime(*time.strptime(value, '%Y-%m-%d %H:%M:%S')[:6])
     
     def db_value(self, value):
-        if value is not None:
-            return '"%s"' % value.strftime('%Y-%m-%d %H:%M:%S')
-        return "NULL"
+        return value or None
 
 
 class IntegerField(Field):
@@ -668,7 +705,7 @@ class ForeignKeyField(IntegerField):
     def lookup_value(self, lookup_type, value):
         if isinstance(value, Model):
             return value.id
-        return value or "NULL"
+        return value or None
 
 
 class BaseModel(type):
@@ -680,7 +717,9 @@ class BaseModel(type):
             
             def __init__(self, model_class):
                 self.model_class = model_class
-                self.database = self.model_class.database
+
+            def get_database(self):
+                return self.model_class.database
             
             def get_field_by_name(self, name):
                 if name in self.fields:
@@ -754,19 +793,19 @@ class Model(object):
     
     @classmethod
     def select(cls, query=None):
-        return SelectQuery(cls.database, cls, query)
+        return SelectQuery(cls, query)
     
     @classmethod
     def update(cls, **query):
-        return UpdateQuery(cls.database, cls, **query)
+        return UpdateQuery(cls, **query)
     
     @classmethod
     def insert(cls, **query):
-        return InsertQuery(cls.database, cls, **query)
+        return InsertQuery(cls, **query)
     
     @classmethod
     def delete(cls, **query):
-        return DeleteQuery(cls.database, cls, **query)
+        return DeleteQuery(cls, **query)
     
     @classmethod            
     def get(cls, **query):
